@@ -1,7 +1,7 @@
 import abc
 import asyncio
 import contextlib
-from typing import AsyncContextManager, AsyncIterator, Callable, Dict, List, Set, Union
+from typing import AsyncContextManager, AsyncIterator, Awaitable, Callable, Dict, List, Set
 
 import yarl
 
@@ -9,20 +9,32 @@ from .base import ClosableResponse, Request, Response
 from .deadline import Deadline
 from .delays_provider import linear_delays
 from .priority import Priority
-from .request_sender import RequestSender
-from .response_classifier import ResponseClassifier, ResponseVerdict
+from .response_classifier import ResponseVerdict
 from .utils import cancel_futures, close, close_futures
 
 
+class SendRequestResult:
+    __slots__ = ("response", "verdict")
+
+    def __init__(self, response: ClosableResponse, verdict: ResponseVerdict):
+        self.response = response
+        self.verdict = verdict
+
+
+SendRequestFunc = Callable[[yarl.URL, Request, Deadline, Priority], Awaitable[SendRequestResult]]
+
+
 class RequestStrategy(abc.ABC):
     __slots__ = ()
 
     @abc.abstractmethod
     def request(
         self,
+        send_request: SendRequestFunc,
+        endpoint: yarl.URL,
         request: Request,
         deadline: Deadline,
-        priority: Priority = Priority.NORMAL,
+        priority: Priority,
     ) -> AsyncContextManager[Response]:
         ...
 
@@ -35,58 +47,35 @@ class MethodBasedStrategy(RequestStrategy):
 
     def request(
         self,
+        send_request: SendRequestFunc,
+        endpoint: yarl.URL,
         request: Request,
         deadline: Deadline,
-        priority: Priority = Priority.NORMAL,
+        priority: Priority,
     ) -> AsyncContextManager[Response]:
-        return self._strategy_by_method[request.method].request(request, deadline, priority)
+        return self._strategy_by_method[request.method].request(send_request, endpoint, request, deadline, priority)
 
 
-class RequestStrategiesFactory:
-    __slots__ = (
-        "_request_sender",
-        "_endpoint",
-        "_response_classifier",
+def sequential_strategy(
+    *, attempts_count: int = 3, delays_provider: Callable[[int], float] = linear_delays()
+) -> RequestStrategy:
+    return SequentialRequestStrategy(
+        attempts_count=attempts_count,
+        delays_provider=delays_provider,
     )
 
-    def __init__(
-        self,
-        request_sender: RequestSender,
-        endpoint: Union[str, yarl.URL],
-        response_classifier: ResponseClassifier,
-    ):
-        self._request_sender = request_sender
-        self._endpoint = yarl.URL(endpoint) if isinstance(endpoint, str) else endpoint
-        self._response_classifier = response_classifier
 
-    def sequential(
-        self, *, attempts_count: int = 3, delays_provider: Callable[[int], float] = linear_delays()
-    ) -> RequestStrategy:
-        return SequentialRequestStrategy(
-            request_sender=self._request_sender,
-            endpoint=self._endpoint,
-            response_classifier=self._response_classifier,
-            attempts_count=attempts_count,
-            delays_provider=delays_provider,
-        )
-
-    def parallel(
-        self, *, attempts_count: int = 3, delays_provider: Callable[[int], float] = linear_delays()
-    ) -> RequestStrategy:
-        return ParallelRequestStrategy(
-            request_sender=self._request_sender,
-            endpoint=self._endpoint,
-            response_classifier=self._response_classifier,
-            attempts_count=attempts_count,
-            delays_provider=delays_provider,
-        )
+def parallel_strategy(
+    *, attempts_count: int = 3, delays_provider: Callable[[int], float] = linear_delays()
+) -> RequestStrategy:
+    return ParallelRequestStrategy(
+        attempts_count=attempts_count,
+        delays_provider=delays_provider,
+    )
 
 
 class SequentialRequestStrategy(RequestStrategy):
     __slots__ = (
-        "_endpoint",
-        "_request_sender",
-        "_response_classifier",
         "_attempts_count",
         "_delays_provider",
     )
@@ -94,31 +83,30 @@ class SequentialRequestStrategy(RequestStrategy):
     def __init__(
         self,
         *,
-        request_sender: RequestSender,
-        endpoint: yarl.URL,
-        response_classifier: ResponseClassifier,
         attempts_count: int,
         delays_provider: Callable[[int], float],
     ):
         if attempts_count < 1:
             raise RuntimeError("Attempts count should be >= 1")
 
-        self._endpoint = endpoint
-        self._request_sender = request_sender
-        self._response_classifier = response_classifier
         self._delays_provider = delays_provider
         self._attempts_count = attempts_count
 
     @contextlib.asynccontextmanager
     async def request(
-        self, request: Request, deadline: Deadline, priority: Priority = Priority.NORMAL
+        self,
+        send_request: SendRequestFunc,
+        endpoint: yarl.URL,
+        request: Request,
+        deadline: Deadline,
+        priority: Priority,
     ) -> AsyncIterator[Response]:
         responses: List[ClosableResponse] = []
         try:
             for attempt in range(self._attempts_count):
-                response = await self._request_sender.send(self._endpoint, request, deadline, priority)
-                responses.append(response)
-                if self._response_classifier.classify(response) == ResponseVerdict.ACCEPT:
+                send_result = await send_request(endpoint, request, deadline, priority)
+                responses.append(send_result.response)
+                if send_result.verdict == ResponseVerdict.ACCEPT:
                     break
                 if attempt + 1 == self._attempts_count:
                     break
@@ -133,9 +121,6 @@ class SequentialRequestStrategy(RequestStrategy):
 
 class ParallelRequestStrategy(RequestStrategy):
     __slots__ = (
-        "_endpoint",
-        "_request_sender",
-        "_response_classifier",
         "_attempts_count",
         "_delays_provider",
     )
@@ -143,36 +128,42 @@ class ParallelRequestStrategy(RequestStrategy):
     def __init__(
         self,
         *,
-        request_sender: RequestSender,
-        endpoint: yarl.URL,
-        response_classifier: ResponseClassifier,
         attempts_count: int,
         delays_provider: Callable[[int], float],
     ):
         if attempts_count < 1:
             raise RuntimeError("Attempts count should be >= 1")
 
-        self._endpoint = endpoint
-        self._request_sender = request_sender
-        self._response_classifier = response_classifier
         self._delays_provider = delays_provider
         self._attempts_count = attempts_count
 
     async def _schedule_request(
-        self, attempt: int, request: Request, deadline: Deadline, priority: Priority = Priority.NORMAL
-    ) -> ClosableResponse:
+        self,
+        attempt: int,
+        send_request: SendRequestFunc,
+        endpoint: yarl.URL,
+        request: Request,
+        deadline: Deadline,
+        priority: Priority,
+    ) -> SendRequestResult:
         if attempt > 0:
             await asyncio.sleep(min(self._delays_provider(attempt), deadline.timeout))
-        return await self._request_sender.send(self._endpoint, request, deadline, priority)
+        return await send_request(endpoint, request, deadline, priority)
 
     @contextlib.asynccontextmanager
     async def request(
-        self, request: Request, deadline: Deadline, priority: Priority = Priority.NORMAL
+        self,
+        send_request: SendRequestFunc,
+        endpoint: yarl.URL,
+        request: Request,
+        deadline: Deadline,
+        priority: Priority,
     ) -> AsyncIterator[Response]:
-        completed_tasks: Set[asyncio.Future[ClosableResponse]] = set()
-        pending_tasks: Set[asyncio.Future[ClosableResponse]] = set()
+        completed_tasks: Set[asyncio.Future[SendRequestResult]] = set()
+        pending_tasks: Set[asyncio.Future[SendRequestResult]] = set()
         for attempt in range(0, self._attempts_count):
-            pending_tasks.add(asyncio.create_task(self._schedule_request(attempt, request, deadline, priority)))
+            schedule_request = self._schedule_request(attempt, send_request, endpoint, request, deadline, priority)
+            pending_tasks.add(asyncio.create_task(schedule_request))
 
         accepted_responses: List[ClosableResponse] = []
         not_accepted_responses: List[ClosableResponse] = []
@@ -184,11 +175,11 @@ class ParallelRequestStrategy(RequestStrategy):
                     )
                     while completed_tasks:
                         completed_task = completed_tasks.pop()
-                        response = await completed_task
-                        if self._response_classifier.classify(response) == ResponseVerdict.ACCEPT:
-                            accepted_responses.append(response)
+                        send_result = await completed_task
+                        if send_result.verdict == ResponseVerdict.ACCEPT:
+                            accepted_responses.append(send_result.response)
                         else:
-                            not_accepted_responses.append(response)
+                            not_accepted_responses.append(send_result.response)
                     if accepted_responses:
                         break
             finally:
@@ -196,5 +187,5 @@ class ParallelRequestStrategy(RequestStrategy):
 
             yield accepted_responses[0] if accepted_responses else not_accepted_responses[0]
         finally:
-            await asyncio.shield(close_futures(pending_tasks | completed_tasks))
+            await asyncio.shield(close_futures(pending_tasks | completed_tasks, lambda x: x.response))
             await asyncio.shield(close(accepted_responses + not_accepted_responses))
